@@ -14,7 +14,7 @@
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
 /**
- * @file backend/taler-merchant-httpd_auditors.c
+ * @file taler-merchant-httpd_auditors.c
  * @brief logic this HTTPD keeps for each exchange we interact with
  * @author Marcello Stanisci
  * @author Christian Grothoff
@@ -61,21 +61,12 @@ static unsigned int nauditors;
  */
 json_t *j_auditors;
 
-
-/**
- * Check if the given @a dk issued by exchange @a mh is audited by
- * an auditor that is acceptable for this merchant. (And if the
- * denomination is not yet expired or something silly like that.)
- *
- * @param mh exchange issuing @a dk
- * @param dk a denomination issued by @a mh
- * @param exchange_trusted #GNUNET_YES if the exchange of @a dk is trusted by config
- * @return #GNUNET_OK if we accept this denomination // FIXME: should return TALER_EC instead!
- */
 int
 TMH_AUDITORS_check_dk (struct TALER_EXCHANGE_Handle *mh,
                        const struct TALER_EXCHANGE_DenomPublicKey *dk,
-                       int exchange_trusted)
+                       bool exchange_trusted,
+                       unsigned int *hc,
+                       enum TALER_ErrorCode *ec)
 {
   const struct TALER_EXCHANGE_Keys *keys;
   const struct TALER_EXCHANGE_AuditorInformation *ai;
@@ -84,16 +75,24 @@ TMH_AUDITORS_check_dk (struct TALER_EXCHANGE_Handle *mh,
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Denomination key offered by client has expired for deposits\n");
+    *hc = MHD_HTTP_GONE;
+    *ec = TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_DENOMINATION_DEPOSIT_EXPIRED;
     return GNUNET_SYSERR; /* expired */
   }
-  if (GNUNET_YES == exchange_trusted)
+  if (exchange_trusted)
+  {
+    *ec = TALER_EC_NONE;
+    *hc = MHD_HTTP_OK;
     return GNUNET_OK;
+  }
   keys = TALER_EXCHANGE_get_keys (mh);
   if (NULL == keys)
   {
     /* this should never happen, keys should have been successfully
        obtained before we even got into this function */
     GNUNET_break (0);
+    *ec = TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE;
+    *hc = MHD_HTTP_INTERNAL_SERVER_ERROR;
     return GNUNET_SYSERR;
   }
   for (unsigned int i = 0; i<keys->num_auditors; i++)
@@ -108,16 +107,21 @@ TMH_AUDITORS_check_dk (struct TALER_EXCHANGE_Handle *mh,
                     "Found supported auditor `%s' (%s)\n",
                     auditors[j].name,
                     TALER_B2S (&auditors[j].public_key));
-
       }
       for (unsigned int k = 0; k<ai->num_denom_keys; k++)
         if (&keys->denom_keys[k] == dk)
+        {
+          *ec = TALER_EC_NONE;
+          *hc = MHD_HTTP_OK;
           return GNUNET_OK;
+        }
     }
   }
   GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
               "Denomination key %s offered by client not audited by any accepted auditor\n",
               GNUNET_h2s (&dk->h_key));
+  *hc = MHD_HTTP_BAD_REQUEST;
+  *ec = TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_DENOMINATION_KEY_AUDITOR_FAILURE;
   return GNUNET_NO;
 }
 
@@ -136,27 +140,38 @@ parse_auditors (void *cls,
   const struct GNUNET_CONFIGURATION_Handle *cfg = cls;
   char *pks;
   struct Auditor auditor;
+  char *currency;
 
   if (0 != strncasecmp (section,
-                        "auditor-",
-                        strlen ("auditor-")))
+                        "merchant-auditor-",
+                        strlen ("merchant-auditor-")))
     return;
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (cfg,
                                              section,
-                                             "NAME",
-                                             &auditor.name))
+                                             "CURRENCY",
+                                             &currency))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                section,
-                               "NAME");
+                               "CURRENCY");
     return;
   }
-  // FIXME: url -> auditor_url
+  if (0 != strcasecmp (currency,
+                       TMH_currency))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Auditor given in section `%s' is for another currency. Skipping.\n",
+                section);
+    GNUNET_free (currency);
+    return;
+  }
+  GNUNET_free (currency);
+  auditor.name = GNUNET_strdup (&section[strlen ("merchant-auditor-")]);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (cfg,
                                              section,
-                                             "URL",
+                                             "AUDITOR_BASE_URL",
                                              &auditor.url))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
@@ -168,12 +183,12 @@ parse_auditors (void *cls,
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (cfg,
                                              section,
-                                             "PUBLIC_KEY",
+                                             "AUDITOR_KEY",
                                              &pks))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                section,
-                               "PUBLIC_KEY");
+                               "AUDITOR_KEY");
     GNUNET_free (auditor.name);
     GNUNET_free (auditor.url);
     return;
@@ -185,8 +200,8 @@ parse_auditors (void *cls,
   {
     GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
                                section,
-                               "PUBLIC_KEY",
-                               "valid public key");
+                               "AUDITOR_KEY",
+                               "need a valid EdDSA public key");
     GNUNET_free (auditor.name);
     GNUNET_free (auditor.url);
     GNUNET_free (pks);
@@ -203,13 +218,6 @@ parse_auditors (void *cls,
 }
 
 
-/**
- * Parses auditor information from the configuration.
- *
- * @param cfg the configuration
- * @return the number of auditors found; #GNUNET_SYSERR upon error in
- *          parsing.
- */
 int
 TMH_AUDITORS_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
@@ -246,7 +254,7 @@ TMH_AUDITORS_done ()
     GNUNET_free (auditors[i].name);
     GNUNET_free (auditors[i].url);
   }
-  GNUNET_free_non_null (auditors);
+  GNUNET_free (auditors);
   auditors = NULL;
   nauditors = 0;
 }
